package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkCellScanMonitorLogService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCellScanMonitorLog;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCellScanMonitorLogRepository;
import com.netflix.discovery.converters.Auto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/30 16:57
 * @Modify:
 **/
@Service
public class MesWorkCellScanMonitorLogServiceImpl implements IMesWorkCellScanMonitorLogService {
    @Autowired
    private MesWorkCellScanMonitorLogRepository monitorLogRao;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;

    /**
     * 根据 MesWorkCellScanMonitorLog 表中 mouldRecordId 开模id 有数据 才去补录
     * 补录数据 是根据MesProductionRecord 中的 开模id相匹配去做
     * @param pageSize
     * @param organizeCode
     */
    @Override
    public void doRecordCellOperationLog(String organizeCode,Integer pageSize) {
        //查询serviceFlag是未处理状态,mouldRecordId不为空的记录，每次查询pageSize 默认50条
        DdlPackBean logPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),"serviceFlag",logPackBean);
        DdlPreparedPack.getIsNotNull("mouldRecordId",logPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"createDatetime"}, logPackBean);
        List<MesWorkCellScanMonitorLog> monitorLogs = monitorLogRao.findByHqlTopWhere(logPackBean, pageSize);

        if (!monitorLogs.isEmpty()) {
            for (MesWorkCellScanMonitorLog monitorLog : monitorLogs) {
                Long mouldRecordId = monitorLog.getMouldRecordId();
                DdlPackBean recordPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getNumEqualPack(mouldRecordId,"mouldRecordId",recordPackBean);
                List<MesProductionRecord> recordList = productionRecordRao.findByHqlWhere(recordPackBean);
                //需要补录如下字段
                if (!recordList.isEmpty()) {
                    String workOrderNoStr = recordList.stream().map(MesProductionRecord::getWorkOrderNo).collect(Collectors.joining(","));
                    String serialNoStr = recordList.stream().map(MesProductionRecord::getSerialNumber).collect(Collectors.joining(","));
                    String productSnStr = recordList.stream().map(MesProductionRecord::getProductSn).collect(Collectors.joining(","));
                    String custSnStr = recordList.stream().map(MesProductionRecord::getCustSn).collect(Collectors.joining(","));
                    String partNoStr = recordList.stream().map(MesProductionRecord::getPartNo).collect(Collectors.joining(","));
                    String partNameStr = recordList.stream().map(MesProductionRecord::getPartName).collect(Collectors.joining(","));
                    monitorLog.setWorkOrderNo(workOrderNoStr);
                    monitorLog.setSerialNumber(serialNoStr);
                    monitorLog.setProductSn(productSnStr);
                    monitorLog.setCustSn(custSnStr);
                    monitorLog.setPartNo(partNoStr);
                    monitorLog.setPartName(partNameStr);
                    monitorLog.setServiceFlag(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                }
                ConvertBean.saveOrUpdate(monitorLog,"JOB");
                monitorLogRao.update(monitorLog);
            }
        }

    }
}
