package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.numberrule.INumberRulePackAttributeStrategy;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNumberRuleStrategyModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesCustomerPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesNumberRuleRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 生成零件条码工步
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductSnGenerateStepService")
public class MesProductSnGenerateStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private MesFirstMouldNoReadStepService firstMouldNoReadStepService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private MesNumberRuleRepository mesNumberRuleRao;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.doHandleCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());

        //存储生产过程上下文对象
        productionProcessContextStepService.saveProductionProcessContext(reqBean, productionProcessContext);

        //从上下文中取出生产线对象
        MesWorkCenter workCenter = productionProcessContext.getWorkCenter();

        //排序线 直接跳过
        if (MesExtEnumUtil.WORK_CENTER_TYPE.SORT.getValue() == workCenter.getCenterType()) return stepResult;

        //已经生成过零件条码
        if (productionDispatchContextStepService.checkProductionPsOutIsExistContext(reqBean)) return stepResult;

        //获取上下文产品加工规则数据信息集合
        List<MesProdRuleContext> prodRuleContextList = productionDispatchContextStepService.getProdRuleDataContext(reqBean);

        if (CollectionUtils.isEmpty(prodRuleContextList)) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "上下文中不存在加工规则信息,请重置工序解决!");

        //获取生产线的当前班组班次信息
        MesProdShiftContext prodShiftContext = productionCustomContextStepService.getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());

        if (null == prodShiftContext) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "上下文中不存在生产开班记录,请重新开班!");

        //获取上下文产出零件数据信息集合
        List<MesProductionPartContext> productionPartContextList = productionDispatchContextStepService.getProductionPartContext(reqBean);

        //获取上下文进料零件条码信息集合
        List<MesProductionPsInContext> productionPsInContextList = productionDispatchContextStepService.getProductionPsInContext(reqBean);

        //获取上下文零件数据信息
        List<String> partNoList = (prodRuleContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getOutPartNo()))).map(MesProdRuleContext::getOutPartNo).collect(Collectors.toList())).stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        firstMouldNoReadStepService.savePartDataMap(reqBean, resultBean, stepResult, partNoList, false);

        //获取上下文加工结果
        String productResult = productionDispatchContextStepService.getProductResultContext(reqBean);

        Map<Integer, MesProductionPartContext> ppMap = CollectionUtils.isEmpty(productionPartContextList) ? null : productionPartContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getForeignKey()))).collect(Collectors.toMap(MesProductionPartContext::getForeignKey, o -> o));
        Map<Integer, MesProductionPsInContext> psMap = CollectionUtils.isEmpty(productionPsInContextList) ? null : productionPsInContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getForeignKey()))).collect(Collectors.toMap(MesProductionPsInContext::getForeignKey, o -> o));
        Map<String, MesPart> partMap = (Map<String, MesPart>) stepResult.getObj();

        //循环生成零件条码，并写入产出零件
        List<MesProductionPsOutContext> productionPsOutContextList = new ArrayList<>();
        prodRuleContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getForeignKey()) && !StringUtils.isEmpty(o.getOutPartNo()))).forEach(o -> {
            MesProductionPsOutContext productionPsOutContext = insertProduceSnData(reqBean, resultBean, stepResult, productionProcessContext, o, ppMap, psMap, partMap, prodShiftContext, productResult);
            if (null != productionPsOutContext) productionPsOutContextList.add(productionPsOutContext);
        });

        //保存上下文产出条码数据信息集合
        productionDispatchContextStepService.saveProductionPsOutContext(reqBean, productionPsOutContextList);

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(MesExtEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), stepResult, "生成零件条码成功!");

    }

    //生成零件条码
    private MesProductionPsOutContext insertProduceSnData(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, MesProductionProcessContext productionProcessContext, MesProdRuleContext prodRuleContext,
                                                          Map<Integer, MesProductionPartContext> ppMap, Map<Integer, MesProductionPsInContext> psMap, Map<String, MesPart> partMap, MesProdShiftContext prodShiftContext, String productResult) {

        //产出零件数据信息
        MesProductionPartContext productionPartContext = !CollectionUtils.isEmpty(ppMap) ? ppMap.get(prodRuleContext.getForeignKey()) : null;
        //料零件条码
        MesProductionPsInContext productionPsInContext = !CollectionUtils.isEmpty(psMap) ? psMap.get(prodRuleContext.getForeignKey()) : null;
        //零件信息
        MesPart part = partMap.get(prodRuleContext.getOutPartNo());

        //加工规则中进料零件与产出零件是否一致
        Boolean isSamePart = isSamePart(prodRuleContext.getInPartNo(), prodRuleContext.getOutPartNo());

        //进出一致, 无需生成
        if (isSamePart) return new MesProductionPsOutContext().copy(produceSnExtService.getProduceSn(reqBean.getOrganizeCode(), null != productionPsInContext ? productionPsInContext.getId() : null), prodRuleContext.getForeignKey());

        //判断存在主条码的时候是否此前已生成当前工位的未知条码
        MesProduceSn produceSn = (null == productionPsInContext || StringUtils.isEmpty(productionPsInContext.getRelateId())) ? null : produceSnExtService.getProduceSn(reqBean.getOrganizeCode(), productionPsInContext.getRelateId());
        if (null == produceSn) produceSn = new MesProduceSn();

        try {
            String sn;
            if(null != productionPsInContext && !StringUtils.isEmpty(productionPsInContext.getProductSn())){
                 sn = productionPsInContext.getProductSn();
            }else {
                String snRule = part.getProductMatchRule();
                //获取条码规则,根据条码规则中的序列号长度截取序列号
                DdlPackBean mesRulePackBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
                DdlPreparedPack.getStringEqualPack(snRule, "ruleCode", mesRulePackBean);
                List<MesNumberRule> ruleList = mesNumberRuleRao.findByHqlWhere(mesRulePackBean);
                if (!ruleList.isEmpty() && !StringUtils.isEmpty(ruleList.get(0).getMethodNameExt())) {
                    MesNumberRule numberRule = ruleList.get(0);
                    String methodNameExt = numberRule.getMethodNameExt();
                    INumberRulePackAttributeStrategy strategy = (INumberRulePackAttributeStrategy)SpringContextsUtil.getBean(methodNameExt);
                    if (strategy == null){
                        execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(),String.format("零件号【%s】的编码规则【%s】生产编码的构造入参方法名未配置!", part.getPartNo(),snRule));
                    }
                    DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
                    DdlPreparedPack.getStringEqualPack(part.getPartNo(), "erpPartNo", ddlPackBean);
                    MesCustomerPart mesCustomerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
                    if (Objects.isNull(mesCustomerPart)) {
                        MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", part.getPartNo());
                    }

                    MesNumberRuleStrategyModel model = new MesNumberRuleStrategyModel();
                    model.setCustPartNo(mesCustomerPart.getCustPartNo());
                    model.setSnRule(snRule);
                    model.setPartNo(part.getPartNo());
                    model.setPartName(part.getPartName());
                    model.setProdLocation(reqBean.getWorkCenterCode());
                    GenSerialNoModel genSerialNoModel = strategy.execute(model);
                    sn = syncFuncService.syncSerialNo(genSerialNoModel, reqBean.getUserInfo(), reqBean.getOrganizeCode(), 1).getResultList().get(0).toString();
                }else {
                    sn=syncFuncService.syncSerialNo(new GenSerialNoModel(getProduceSnNumberRule(reqBean, resultBean, part)).partNo(part.getPartNo()), reqBean.getUserInfo(), reqBean.getOrganizeCode(), 1).getResultList().get(0).toString();
                }
            }
            produceSn.setProductSn(sn);

            produceSn.setSerialNumber(syncFuncService.syncSerialNo(new GenSerialNoModel(MesPcnExtConstWords.DEFAULT_SERIAL_NUMBER_RULE).prodLocation(reqBean.getWorkCenterCode()), reqBean.getUserInfo(), reqBean.getOrganizeCode(), 1).getResultList().get(0).toString());

        } catch (ImppBusiException e) {
            execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), e.getErrorDetail());
        }

        produceSn.setCustSn(produceSn.getProductSn());
        produceSn.setQty(new Double(1));

        //未知状态的需要重置工单相关字段
        produceSn.setWorkOrderNo((null != productionPartContext && !StringUtils.isEmpty(productionPartContext.getWorkOrderNo())) ? productionPartContext.getWorkOrderNo() : null);
        produceSn.setCustCode((null != productionPartContext && !StringUtils.isEmpty(productionPartContext.getWorkOrderNo())) ? productionPartContext.getCustCode() : null);
        produceSn.setCustPartNo((null != productionPartContext && !StringUtils.isEmpty(productionPartContext.getWorkOrderNo())) ? productionPartContext.getCustPartNo() : null);

        produceSn.setAreaCode(reqBean.getAreaCode());
        produceSn.setWorkCenterCode(reqBean.getWorkCenterCode());
        produceSn.setWorkCellCode(reqBean.getWorkCellCode());

        produceSn.setPartNo(part.getPartNo());
        produceSn.setPartName(part.getPartName());
        produceSn.setProcessLabelTemplate(part.getProcessLabelTemplate());
        produceSn.setProdLabelTemplate(part.getProductLabelTemplate());
        produceSn.setCustLabelTemplate(part.getCustLabelTemplate());

        produceSn.setSnType(MesExtEnumUtil.PRODUCE_SN_TYPE.STANDARD.getValue());
        produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.UNKNOW.getValue());
        produceSn.setQcStatus(!StringUtils.isEmpty(productResult) ? Integer.valueOf(productResult) : MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());

        produceSn.setShiftCode(prodShiftContext.getShiftCode());
        produceSn.setShiftGroup(prodShiftContext.getShiftGroup());

        produceSn.setPrintCount(MesPcnExtConstWords.ZERO);
        produceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.UNPRINT.getValue());

        produceSn.setOrganizeCode(reqBean.getOrganizeCode());

        if (StringUtils.isEmpty(produceSn.getId())) ConvertBean.serviceModelInitialize(produceSn, reqBean.getUserInfo());
        else ConvertBean.serviceModelUpdate(produceSn, reqBean.getUserInfo());

        produceSn.setInWorkCenterTime(produceSn.getModifyDatetime());
        produceSn.setLotNo(produceSn.getModifyDatetime().substring(0, 10));

        if (StringUtils.isEmpty(produceSn.getId())) produceSn = produceSnRepository.insert(produceSn);
        else produceSnRepository.save(produceSn);

        return new MesProductionPsOutContext().copy(produceSn, prodRuleContext.getForeignKey()).isCalcCompleteQty(produceSn.getWorkOrderNo());

    }

    private String getProduceSnNumberRule(StationRequestBean reqBean, StationResultBean resultBean, MesPart part) {
        if (StringUtils.isEmpty(part.getProductMatchRule())) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("请检查零件信息,零件[%s]信息未维护零件条码编码规则!", part.getPartNo()));
        return part.getProductMatchRule();
    }

    private Boolean isSamePart(String inPartNo, String outPartNo) {
        if (!StringUtils.isEmpty(inPartNo) && inPartNo.equals(outPartNo)) return true;
        else return false;
    }

}
