package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 保存零件条码信息工步
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductSnSaveStepService")
public class MesProductSnSaveStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private MesFirstMouldNoReadStepService firstMouldNoReadStepService;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.doHandleCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());

        //存储生产过程上下文对象
        productionProcessContextStepService.saveProductionProcessContext(reqBean, productionProcessContext);

        //获取上下文产出条码数据信息集合
        List<MesProductionPsOutContext> productionPsOutContextList = productionDispatchContextStepService.getProductionPsOutContext(reqBean);

        if (CollectionUtils.isEmpty(productionPsOutContextList)) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "上下文中不存在产出零件条码信息,请重置工序解决!");

        //获取生产线的当前班组班次信息
        MesProdShiftContext prodShiftContext = productionCustomContextStepService.getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());

        if (null == prodShiftContext) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "上下文中不存在生产开班记录,请重新开班!");

        //获取上下文零件数据信息
        List<String> partNoList = (productionPsOutContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getPartNo()))).map(MesProductionPsOutContext::getPartNo).collect(Collectors.toList())).stream().filter(o -> !StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
        firstMouldNoReadStepService.savePartDataMap(reqBean, resultBean, stepResult, partNoList, false);

        //获取上下文加工结果
        String productResult = productionDispatchContextStepService.getProductResultContext(reqBean);
        Map<String, MesPart> partMap = (Map<String, MesPart>) stepResult.getObj();

        productionPsOutContextList.stream().filter(o -> null != o).forEach(o -> o.copy(saveProduceSnData(reqBean, resultBean, stepResult, productionProcessContext, o, partMap, prodShiftContext, productResult)));

        //保存上下文产出条码数据信息集合
        productionDispatchContextStepService.saveProductionPsOutContext(reqBean, productionPsOutContextList);

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(MesExtEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), stepResult, "保存零件条码成功!");

    }

    private MesProduceSn saveProduceSnData(StationRequestBean reqBean, StationResultBean resultBean, StepResult stepResult, MesProductionProcessContext productionProcessContext, MesProductionPsOutContext productionPsOutContext, Map<String, MesPart> partMap, MesProdShiftContext prodShiftContext, String productResult) {

        //根据零件条码ID查询零件条码信息
        MesProduceSn produceSn = produceSnExtService.getProduceSn(reqBean.getOrganizeCode(), productionPsOutContext.getId());

        if (null == produceSn) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("请检查零件条码信息,产出零件条码[%s]ID[%s]信息不存在!", productionPsOutContext.getProductSn(), productionPsOutContext.getId()));

        //零件信息
        MesPart part = partMap.get(produceSn.getPartNo());

        produceSn.setAreaCode(reqBean.getAreaCode());
        produceSn.setWorkCenterCode(reqBean.getWorkCenterCode());
        produceSn.setWorkCellCode(reqBean.getWorkCellCode());

        produceSn.setRouteCode(reqBean.getRouteCode());
        produceSn.setProcessCode(reqBean.getProcessCode());
        produceSn.setCraftCode(productionProcessContext.getCraftCode());
        //TODO 获取下个工艺代码
        //produceSn.setNextCraftCode();
        //TODO 重新覆盖打印模版

        produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue());
        produceSn.setQcStatus(!StringUtils.isEmpty(productResult) ? Integer.valueOf(productResult) : MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());

        produceSn.setShiftCode(prodShiftContext.getShiftCode());
        produceSn.setShiftGroup(prodShiftContext.getShiftGroup());

        produceSn.setProcessLabelTemplate(part.getProcessLabelTemplate());
        produceSn.setProdLabelTemplate(part.getProductLabelTemplate());
        produceSn.setCustLabelTemplate(part.getCustLabelTemplate());

        ConvertBean.serviceModelUpdate(produceSn, reqBean.getUserInfo());
        produceSn.setOutWorkCenterTime(produceSn.getModifyDatetime());
        produceSn.setLotNo(produceSn.getModifyDatetime().substring(0, 10));

        return produceSnRepository.save(produceSn);

    }

}
